import ace, { type Ace } from 'ace-builds';
import {
  defineComponent,
  markRaw,
  watch,
  reactive,
  onBeforeUnmount,
  onMounted,
  ref,
  unref,
  computed,
  nextTick
} from 'vue';
import './config';
import ResizeObserver from 'resize-observer-polyfill';
import { basicEmits } from './emits';
import { basicProps } from './props';
import type { AceEditorState, AceEditorMethods, AceEditorProps } from './types';
import { deepMerge } from '/@/utils';
import { useAppStore } from '/@/store/modules/app';
import { isFunction } from '/@/utils/is';

export default defineComponent({
  name: 'AceEditor',
  props: basicProps,
  emits: basicEmits,
  setup(props, { emit, attrs }) {
    const aceEditorElRef = ref();
    const propsRef = ref<Partial<AceEditorProps>>();
    const appStore = useAppStore();

    const state = reactive<AceEditorState>({
      _editor: undefined!,
      _ro: undefined!,
      _contentBackup: '',
      _isSettingContent: false,
    });

    const getProps = computed((): AceEditorProps => {
      return {
        ...props,
        ...(unref(propsRef) as any),
      };
    });

    onMounted(async () => {
      await nextTick();
      const editor = state._editor = markRaw(ace.edit(aceEditorElRef.value, {
        placeholder: props.placeholder,
        readOnly: props.readonly,
        value: props.value,
        mode: 'ace/mode/' + props.lang,
        theme: 'ace/theme/' + props.theme,
        wrap: props.wrap,
        printMargin: props.printMargin,
        useWorker: false,
        minLines: props.minLines,
        maxLines: props.maxLines,
        ...props.options,
      }));
      setTheme();
      state._contentBackup = props.value;
      state._isSettingContent = false;
      editor.on('change', () => {
        // ref: https://github.com/CarterLi/vue3-ace-editor/issues/11
        if (state._isSettingContent) return;
        const content = editor.getValue();
        state._contentBackup = content;
        emit('update:value', content);
        emit('change', content);
        // 解决通过useAceEditor注册onChange时不起作用的问题
        const { onChange } = unref(getProps);
        onChange && isFunction(onChange) && onChange.call(undefined, content);
      });
      // Event Binding
      editor.on('blur', (e: Event) => emit('blur', e));
      editor.on('input', () => emit('input'));
      editor.on('changeSelectionStyle', (obj: { data: string }) => emit('changeSelectionStyle', obj));
      editor.on('changeSession', (obj: { session: Ace.EditSession, oldSession: Ace.EditSession }) => emit('changeSelectionStyle', obj));
      editor.on('copy', (obj: { text: string }) => emit('copy', obj));
      editor.on('focus', (e: Event) => emit('focus', e));
      editor.on('paste', (obj: { text: string }) => emit('paste', obj));

      state._ro = new ResizeObserver(() => editor.resize());
      state._ro.observe(aceEditorElRef.value);
      emit('register', aceEditorMethods);
    });

    onBeforeUnmount(() => {
      state._ro?.disconnect();
      state._editor?.destroy();
    });

    watch(() => appStore.getDarkMode, async () => {
        setTheme();
      },
      {
        immediate: true,
      },
    );

    watch(() => unref(getProps).value, (val) => {
      if (state._contentBackup !== val) {
        try {
          state._isSettingContent = true;
          state._editor.setValue(val, 1);
        } finally {
          state._isSettingContent = false;
        }
        state._contentBackup = val;
      }
    });

    watch(() => unref(getProps).theme, (val) => {
      state._editor.setTheme('ace/theme/' + val);
    });

    watch(() => unref(getProps).options, (val: Partial<Ace.EditorOptions>) => {
      state._editor.setOptions(val);
    });

    watch(() => unref(getProps).readonly, (val) => {
      state._editor.setReadOnly(val);
    });

    watch(() => unref(getProps).placeholder, (val) => {
      state._editor.setOption('placeholder', val);
    });

    watch(() => unref(getProps).wrap, (val) => {
      state._editor.setWrapBehavioursEnabled(val);
    });

    watch(() => unref(getProps).printMargin, (val) => {
      state._editor.setOption('printMargin', val);
    });

    watch(() => unref(getProps).lang, (val) => {
      state._editor.setOption('mode', 'ace/mode/' + val);
    });

    watch(() => unref(getProps).minLines, (val) => {
      state._editor.setOption('minLines', val);
    });

    watch(() => unref(getProps).maxLines, (val) => {
      state._editor.setOption('maxLines', val);
    });

    function setTheme() {
      const theme = appStore.getDarkMode === 'light' ? 'chrome' : 'monokai';
      state._editor?.setTheme('ace/theme/' + theme);
    }

    function setProps(props: Partial<AceEditorProps>) {
      propsRef.value = deepMerge(unref(propsRef) || {}, props);
    }

    function focus() {
      state._editor.focus();
    }

    function blur() {
      state._editor.blur();
    }

    function selectAll() {
      state._editor.selectAll();
    }

    function getAceInstance(): Ace.Editor {
      return state._editor;
    }

    const aceEditorMethods: AceEditorMethods = {
      setProps,
      focus,
      blur,
      selectAll,
      getAceInstance
    };

    return {
      aceEditorElRef,
      setProps,
      focus,
      blur,
      selectAll,
      getAceInstance,
    };
  },
  render() {
    return (
      <div class="!h-full w-full" ref="aceEditorElRef" { ...this.$attrs }></div>
    );
  }
});
