import { AceEditorProps, AceEditorMethods, AceEditorInstance } from '/@/components/AceEditor/src/types';
import { onUnmounted, ref, unref, watch, type WatchStopHandle } from 'vue';
import { isProdMode } from '/@/utils/env';
import { getDynamicProps } from '/@/utils';
import { error } from '/@/utils/log';
import type { DynamicProps } from '/#/utils';

type Props = Partial<DynamicProps<AceEditorProps>>;

export function useAceEditor(aceEditorProps?: Props): [
  (instance: AceEditorInstance) => void,
  AceEditorMethods
] {

  const aceEditorRef = ref<Nullable<AceEditorInstance>>(null);
  const loadedRef = ref<Nullable<boolean>>(false);

  let stopWatch: WatchStopHandle;

  async function register(instance: AceEditorInstance) {
    isProdMode() &&
    onUnmounted(() => {
      aceEditorRef.value = null;
      loadedRef.value = null;
    });

    // 防止同一个组件重复注册
    if (unref(loadedRef) && isProdMode() && instance === unref(aceEditorRef)) return;

    aceEditorRef.value = instance;
    aceEditorProps && instance.setProps(getDynamicProps(aceEditorProps));
    loadedRef.value = true;

    stopWatch?.();

    stopWatch = watch(
      () => aceEditorProps,
      () => {
        aceEditorProps && instance.setProps(getDynamicProps(aceEditorProps));
      },
      {
        immediate: true,
        deep: true,
      },
    );
  }

  function getAceEditorInstance(): AceEditorInstance {
    const aceEditor = unref(aceEditorRef);
    if (!aceEditor) {
      error(
        'The AceEditor instance has not been obtained yet, please make sure the AceEditor is presented when performing the AceEditor operation!',
      );
    }
    return aceEditor as AceEditorInstance;
  }

  const methods: AceEditorMethods = {
    focus: () => {
      getAceEditorInstance().focus();
    },
    blur: () => {
      getAceEditorInstance().blur();
    },
    selectAll: () => {
      getAceEditorInstance().selectAll();
    },
    setProps: (props: Partial<AceEditorProps>) => {
      getAceEditorInstance().setProps(props);
    },
    getAceInstance: () => {
     return  getAceEditorInstance().getAceInstance();
    }
  };

  return [register, methods];
}
