/**
 * @program: kicc-ui
 * @description: 权限导航守卫
 * 校验当前用户是否有权限能够进入系统
 * @author: wangxiang4
 * @create: 2022/4/9
 */

import type { Router, RouteRecordRaw } from 'vue-router';
import { usePermissionStoreWithOut } from '/@/store/modules/permission';
import { PageEnum, SsoLoginPageEnum } from '/@/enums/pageEnum';
import { useUserStoreWithOut } from '/@/store/modules/user';
import { RootRoute } from '/@/router/routes';
import { PAGE_NOT_FOUND_ROUTE } from '/@/router/routes/basic';
import { setObjToUrlParams } from '/@/utils';
import { AesEncryption } from '/@/utils/cipher';
import { cacheCipher } from '/@/settings/encryptionSetting';
import axios from 'axios';
import { useGlobSetting } from '/@/hooks/setting';
import { Api } from '/@/api/platform/core/controller/user';

type BasicPageEnum = PageEnum | SsoLoginPageEnum;
const LOGIN_PATH = PageEnum.BASE_LOGIN;
const ROOT_PATH = RootRoute.path;
const ssoPathList: SsoLoginPageEnum[] = Object.values(SsoLoginPageEnum);
/** 设置路由白名单 */
const whitePathList: BasicPageEnum[] = [ LOGIN_PATH, ...ssoPathList ];
const encryption = new AesEncryption({ key: cacheCipher.key, iv: cacheCipher.iv });
const globSetting = useGlobSetting();

export function createPermissionGuard(router: Router) {
  const userStore = useUserStoreWithOut();
  const permissionStore = usePermissionStoreWithOut();
  router.beforeEach(async (to, from, next) => {
    // 校验token权限信息
    const token = userStore.getAccessToken;
    // sso重定向转发token进行登陆
    if(ssoPathList.includes(to.path as SsoLoginPageEnum) && token) {
      try {
        // 校验token是否过期
        await axios.get(`${globSetting.apiUrl}${globSetting.urlPrefix}${Api.checkToken}`, {params: {token}});
        return window.location.replace(to.query?.redirect ? setObjToUrlParams(to.query?.redirect as string, {
          ticket: encryption.encryptByAES(token)
        }) : '/');
      } catch (e) {}
    }

    if (
      // 如果指定首页路径数据不为空就代表已经登录直接放过
      from.path === ROOT_PATH &&
      to.path === PageEnum.BASE_HOME &&
      userStore.getUserInfo.homePath &&
      userStore.getUserInfo.homePath !== PageEnum.BASE_HOME
    ) {
      return next(userStore.getUserInfo.homePath);
    }

    // 放过白名单路由
    if (whitePathList.includes(to.path as PageEnum)) {
      if (to.path === LOGIN_PATH && token) {
        const isSessionTimeout = userStore.getSessionTimeout;
        try {
          await userStore.afterLoginAction();
          if (!isSessionTimeout) {
            next((to.query?.redirect as string) || '/');
            return;
          }
        } catch {}
      }
      return next();
    }
    if (!token) {
      // 重定向登录页面
      const redirectLogin: { path: string; replace: boolean; query?: Recordable<string>; } = { path: LOGIN_PATH, replace: true };
      // 补充拦截路由信息
      if (to.path) {
        redirectLogin.query = {
          ...redirectLogin.query,
          redirect: to.path,
        };
      }
      return next(redirectLogin);
    }

    if (
      // 处理避免登录后跳转到404页面
      from.path === LOGIN_PATH &&
      to.name === PAGE_NOT_FOUND_ROUTE.name &&
      to.fullPath !== (userStore.getUserInfo.homePath || PageEnum.BASE_HOME)
    ) {
      next(userStore.getUserInfo.homePath || PageEnum.BASE_HOME);
      return;
    }
    // 页面刷新重新拉取用户信息数据(修改redis内多租户或切换用户角色会改变当前用户数据)
    if (userStore.getLastUpdateTime === 0) {
      try {
        await userStore.getUserInfoAction();
      } catch (err) {
        console.info(err);
        next();
      }
    }
    // 检测是否构建完路由与菜单
    if (permissionStore.getIsDynamicAddedRoute) {
      return next();
    }
    // 构建路由与菜单
    const routes = await permissionStore.buildRoutesAction();
    routes.forEach((route) => router.addRoute(route as unknown as RouteRecordRaw));
    permissionStore.setDynamicAddedRoute(true);
    // 动态添加路由后,此处应当重定向到fullPath,否则会加载404页面内容,添加query以免丢失
    if (to.name === PAGE_NOT_FOUND_ROUTE.name) {
      next({ path: to.fullPath, replace: true, query: to.query });
    } else {
      const redirectPath = (from.query.redirect || to.path) as string;
      const redirect = decodeURIComponent(redirectPath);
      const nextData = to.path === redirect ? { ...to, replace: true } : { path: redirect };
      next(nextData);
    }
  });
}
