/**
 * @program: kicc-ui
 * @description: 通用工具类
 * 注意: 如果同一类的工具类很多,建议创建文件存放到这个文件中,方便工具分类,与后期维护
 * @author: wangxiang4
 * @create: 2022/4/8
 */

import type {RouteLocationNormalized, RouteRecordNormalized} from 'vue-router';
import type {App, Component} from 'vue';
import {unref} from 'vue';
import {isObject, isUrl} from '/@/utils/is';
import {cloneDeep, isEmpty} from 'lodash-es';
import {isDevMode} from '/@/utils/env';
import {useGlobSetting} from '/@/hooks/setting';

const { apiUrl } = useGlobSetting();
export const noop = () => {};

/**
 * 设置ui挂载节点
 */
export function getPopupContainer(node?: HTMLElement): HTMLElement {
  return (node?.parentNode as HTMLElement) ?? document.body;
}

/**
 * 将对象作为参数添加到 URL
 * @param baseUrl url
 * @param obj
 * @returns {string}
 * eg:
 *  let obj = {a: '3', b: '4'}
 *  setObjToUrlParams('cn.vuejs.org', obj)
 *  ==>cn.vuejs.org?a=3&b=4
 */
export function setObjToUrlParams(baseUrl: string, obj: any): string {
  let parameters = '';
  for (const key in obj) {
    parameters += key + '=' + encodeURIComponent(obj[key]) + '&';
  }
  parameters = parameters.replace(/&$/, '');
  return /\?$/.test(baseUrl) ? baseUrl + parameters : baseUrl.replace(/\/?$/, '?') + parameters;
}

export function deepMerge<T = any>(src: any = {}, target: any = {}): T {
  let key: string;
  const res: any = cloneDeep(src);
  for (key in target) {
    res[key] = isObject(res[key]) ? deepMerge(res[key], target[key]) : target[key];
  }
  return res;
}

export function openWindow(
  url: string,
  opt?: { target?: TargetContext | string; noopener?: boolean; noreferrer?: boolean }
) {
  const { target = '__blank', noopener = true, noreferrer = true } = opt || {};
  const feature: string[] = [];

  noopener && feature.push('noopener=yes');
  noreferrer && feature.push('noreferrer=yes');

  window.open(url, target, feature.join(','));
}

// 动态使用钩子道具
export function getDynamicProps<T, U>(props: T): Partial<U> {
  const ret: Recordable = {};

  Object.keys(props!).map((key) => {
    ret[key] = unref((props as Recordable)[key]);
  });

  return ret as Partial<U>;
}

export function getRawRoute(route: RouteLocationNormalized): RouteLocationNormalized {
  if (!route) return route;
  const { matched, ...opt } = route;
  return {
    ...opt,
    matched: (matched
      ? matched.map((item) => ({
          meta: item.meta,
          name: item.name,
          path: item.path,
        }))
      : undefined) as RouteRecordNormalized[],
  };
}

// https://github.com/vant-ui/vant/issues/8302
type EventShim = {
  new (...args: any[]): {
    $props: {
      onClick?: (...args: any[]) => void;
    };
  };
};

export type WithInstall<T> = T & {
  install(app: App): void;
} & EventShim;

export type CustomComponent = Component & { displayName?: string };

export const withInstall = <T extends CustomComponent>(component: T, alias?: string) => {
  (component as Record<string, unknown>).install = (app: App) => {
    const compName = component.name || component.displayName;
    if (!compName) return;
    app.component(compName, component);
    if (alias) {
      app.config.globalProperties[alias] = component;
    }
  };
  return component as WithInstall<T>;
};


/** 查询指定集合名称 */
export const findListNameById = (id: string, list: any[], options: Partial<{ idField: string, nameField: string, childrenField: string }> = {}) => {
  // 设置默认配置
  options = Object.assign({ idField: 'id', nameField: 'name', childrenField: 'children' }, options);
  for (let i = 0; i < list.length; i++) {
    if (list[i][options.idField!] === id) {
      return list[i][options.nameField!];
    }
    if (!isEmpty(list[i][options.childrenField!])) {
      return findListNameById(id, list[i][options.childrenField!],options);
    }
  }
  return '';
};

/** 获取后端微服务代理地址 */
export function getCloudProxyUrl(): string {
  let baseUrl = `${window.location.origin}${apiUrl}`;
  if (isDevMode()) {
    // 开发环境支持公网地址,确保开发环境下支持多人共享文件预览
    const proxy: [string[]] = JSON.parse(import.meta.env.VITE_PROXY);
    const api = proxy.find(item => item[0] == apiUrl) || [];
    baseUrl = api[1];
  }
  return baseUrl;
}

/** 获取OSS代理地址 */
export const getOSSProxyUrl = (url: string) => isUrl(url) ? url :  `${apiUrl}${url}`;

/** 获取授权客户端 */
export function getAuthClient(clientId: string): string[] {
  const authClient: [string[]] = JSON.parse(import.meta.env['VITE_AUTH_CLIENT']);
  return authClient.find(item => item[0] == clientId) || [];
}
